@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2005, 2007, 2010-2012, 2014-2016 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2019 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -126,7 +125,9 @@ extern mu_filter_record_t mu_iconv_filter;
 extern mu_filter_record_t mu_c_escape_filter;
 extern mu_filter_record_t mu_htmlent_filter;
 extern mu_filter_record_t mu_xml_filter;
-  
+extern mu_filter_record_t mu_percent_filter;
+extern mu_filter_record_t mu_dq_filter;
+
 enum mu_iconv_fallback_mode
   {
     mu_fallback_none,
