@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2005-2007, 2010-2012, 2014-2016 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2019 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -73,6 +72,8 @@ extern int  mu_mailbox_expunge         (mu_mailbox_t);
 extern int  mu_mailbox_sync            (mu_mailbox_t);  
 extern int  mu_mailbox_save_attributes (mu_mailbox_t) MU_DEPRECATED;
 
+extern int  mu_mailbox_attach_ticket (mu_mailbox_t mbox);
+
 #define MU_UIDL_LENGTH 70
 #define MU_UIDL_BUFFER_SIZE (MU_UIDL_LENGTH+1)
 				       
