@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2003, 2004, 2005,
-   2007 Free Software Foundation, Inc.
+   Copyright (C) 1999-2019 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -13,9 +12,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -27,29 +25,23 @@
 #include <sys/stat.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
 #include <errno.h>
 #include <unistd.h>
 #include <dirent.h>
 
-#include <url0.h>
-#include <folder0.h>
-#include <registrar0.h>
-#include <amd.h>
-#include <mailutils/mutil.h>
-
-static int
-_mh_folder_init (mu_folder_t folder MU_ARG_UNUSED)
-{
-  return 0;
-}
+#include <mailutils/sys/url.h>
+#include <mailutils/sys/folder.h>
+#include <mailutils/sys/registrar.h>
+#include <mailutils/sys/amd.h>
+#include <mailutils/util.h>
+#include <mailutils/cctype.h>
 
 /* Check if NAME is a valid MH message name */
 static int
 mh_message_name_p (const char *name)
 {
   for ( ; *name; name++)
-    if (!isdigit (*name))
+    if (!mu_isdigit (*name))
       return 0;
   return 1;
 }
@@ -91,13 +83,11 @@ static int
 _mh_is_scheme (mu_record_t record, mu_url_t url, int flags)
 {
   int rc = 0;
+  int scheme_matched = mu_url_is_scheme (url, record->scheme);
   
-  if (mu_url_is_scheme (url, record->scheme))
-    return MU_FOLDER_ATTRIBUTE_ALL & flags;
-
-  if (mu_scheme_autodetect_p (url))
+  if (scheme_matched || mu_scheme_autodetect_p (url))
     {
-      /* Attemp auto-detection */
+      /* Attempt auto-detection */
       const char *path;
       struct stat st;
       
@@ -105,17 +95,40 @@ _mh_is_scheme (mu_record_t record, mu_url_t url, int flags)
         return 0;
 
       if (stat (path, &st) < 0)
-	return 0; /* mu_mailbox_open will complain*/
-
+	{
+	  if (errno == ENOENT && scheme_matched)
+	    return MU_FOLDER_ATTRIBUTE_ALL & flags; 
+	  return 0; /* mu_mailbox_open will complain*/
+	}
+      
       if (!S_ISDIR (st.st_mode))
 	return 0;
 
-      rc |= (MU_FOLDER_ATTRIBUTE_DIRECTORY & flags);
+      if (scheme_matched)
+	rc = MU_FOLDER_ATTRIBUTE_ALL;
+      else
+	{
+	  rc |= MU_FOLDER_ATTRIBUTE_DIRECTORY;
       
-      if ((flags & MU_FOLDER_ATTRIBUTE_FILE) && mh_dir_p (path))
-        return rc | MU_FOLDER_ATTRIBUTE_FILE;
+	  if ((flags & MU_FOLDER_ATTRIBUTE_FILE) && mh_dir_p (path))
+	    rc |= MU_FOLDER_ATTRIBUTE_FILE;
+	}
     }
 
+  return rc & flags;
+}
+
+static int
+_mh_list_p (mu_record_t record, const char *name, int flags MU_ARG_UNUSED)
+{
+  if (name[0] == ',' ||
+      ((strlen (name) > 3) &&
+       (memcmp (name, ".mh", 3) == 0 || memcmp (name, ".mu", 3) == 0)))
+    return 0;
+
+  for (; *name; name++)
+    if (!mu_isdigit (*name))
+      return 1;
   return 0;
 }
 
@@ -123,21 +136,25 @@ static struct _mu_record _mh_record =
 {
   MU_MH_PRIO,
   MU_MH_SCHEME,
+  MU_RECORD_LOCAL,
+  MU_URL_SCHEME | MU_URL_PATH | MU_URL_PARAM,
+  MU_URL_PATH,
   mu_url_expand_path, /* Url init.  */
   _mailbox_mh_init, /* Mailbox init.  */
   NULL, /* Mailer init.  */
-  _mh_folder_init, /* Folder init.  */
+  _mu_fsfolder_init, /* Folder init.  */
   NULL, /* back pointer.  */
   _mh_is_scheme, /* _is_scheme method.  */
   NULL, /* _get_url method.  */
   NULL, /* _get_mailbox method.  */
   NULL, /* _get_mailer method.  */
-  NULL  /* _get_folder method.  */
+  NULL, /* _get_folder method.  */
+  _mh_list_p
 };
 mu_record_t mu_mh_record = &_mh_record;
 
 #else
 #include <stdio.h>
-#include <registrar0.h>
+#include <mailutils/sys/registrar.h>
 mu_record_t mu_mh_record = NULL;
 #endif
