@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2007, 2010 Free Software Foundation, Inc.
+   Copyright (C) 2007-2019 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "maidag.h"
 
@@ -70,11 +68,13 @@ maidag_error (const char *fmt, ...)
 
   guess_retval (errno);
   va_start (ap, fmt);
-  if (log_to_stderr)
+  if (!mu_log_syslog)
     {
       vfprintf (stderr, fmt, ap);
       fputc ('\n', stderr);
     }
+  va_end (ap);
+  va_start (ap, fmt);
   mu_verror (fmt, ap);
   va_end (ap);
 }
