@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2003, 2005-2007, 2009-2012, 2014-2017 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2019 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,19 +28,10 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <fnmatch.h>
-
-char mh_list_format[] = 
-  "%4(msg)"
-  "%<(cur)+%| %>"
-  "%<{replied}-%?{encrypted}E%| %>"
-  "%02(mon{date})/%02(mday{date})"
-  "%<{date} %|*%>"
-  "%<(mymbox{from})%<{to}To:%14(decode(friendly{to}))%>%>"
-  "%<(zero)%17(decode(friendly{from}))%>"
-  "  %(decode{subject})%<{body}<<%{body}>>%>";
+#include <sys/ioctl.h>
 
 void
-mh_init ()
+mh_init (void)
 {
   mu_stdstream_setup (MU_STDSTREAM_RESET_NONE);
   
@@ -53,75 +43,11 @@ mh_init ()
 }
 
 void
-mh_init2 ()
+mh_init2 (void)
 {
   mh_current_folder ();
 }
 
-int
-mh_read_formfile (char const *name, char **pformat)
-{
-  FILE *fp;
-  struct stat st;
-  char *ptr;
-  size_t off = 0;
-  char *format_str;
-  char *file_name;
-  int rc;
-  
-  rc = mh_find_file (name, &file_name);
-  if (rc)
-    {
-      mu_error (_("cannot access format file %s: %s"), name, strerror (rc));
-      return -1;
-    }
-  
-  if (stat (file_name, &st))
-    {
-      mu_error (_("cannot stat format file %s: %s"), file_name,
-		strerror (errno));
-      free (file_name);
-      return -1;
-    }
-  
-  fp = fopen (file_name, "r");
-  if (!fp)
-    {
-      mu_error (_("cannot open format file %s: %s"), file_name,
-		strerror (errno));
-      free (file_name);
-      return -1;
-    }
-  free (file_name);
-  
-  format_str = mu_alloc (st.st_size+1);
-  while ((ptr = fgets (format_str + off, st.st_size - off + 1, fp)) != NULL)
-    {
-      int len = strlen (ptr);
-      if (len == 0)
-	break;
-
-      if (*ptr == '%' && ptr[1] == ';')
-	continue;
-      
-      if (len > 0 && ptr[len-1] == '\n')
-	{
-	  if (ptr[len-2] == '\\')
-	    {
-	      len -= 2;
-	      ptr[len] = 0;
-	    }
-	}
-      off += len;
-    }
-  if (off > 0 && format_str[off-1] == '\n')
-    off--;
-  format_str[off] = 0;
-  fclose (fp);
-  *pformat = format_str;
-  return 0;
-}
-
 void
 mh_err_memory (int fatal)
 {
@@ -130,108 +56,257 @@ mh_err_memory (int fatal)
     abort ();
 }
 
-static char *my_name;
-static char *my_email;
-
-static char *
-mh_get_my_name (void)
+static mu_address_t
+mh_local_mailbox (void)
 {
-  if (!my_name)
+  static mu_address_t local_mailbox;
+  if (!local_mailbox)
     {
-      struct passwd *pw = getpwuid (getuid ());
-      if (!pw)
-	{
-	  mu_error (_("cannot determine my username"));
-	  my_name = mu_strdup ("unknown");
-	}
-      else
-	my_name = mu_strdup (pw->pw_name);
+      const char *p = mh_global_profile_get ("Local-Mailbox", NULL);
+      if (!p)
+	p = mu_get_user_email (NULL);
+      mu_address_create (&local_mailbox, p);
     }
-  return my_name;
+  return local_mailbox;
 }
 
-char *
+char const *
+mh_get_my_user_name (void)
+{
+  mu_address_t addr = mh_local_mailbox ();
+  char const *s;
+  MU_ASSERT (mu_address_sget_local_part (addr, 1, &s));
+  return s;
+}
+
+char const *
+mh_get_my_real_name (void)
+{
+  mu_address_t addr = mh_local_mailbox ();
+  char const *s;
+  if (mu_address_sget_personal (addr, 1, &s))
+    return NULL;
+  return s;
+}
+
+char const *
 mh_my_email (void)
 {
-  char *username = mh_get_my_name ();
-  if (!my_email)
-    {  
-      const char *p = mh_global_profile_get ("Local-Mailbox", NULL);
-      if (p)
-	my_email = mu_strdup (p);
-      else
-	my_email = mu_get_user_email (username);
-    }
-  return my_email;
+  mu_address_t addr = mh_local_mailbox ();
+  char const *s;
+  MU_ASSERT (mu_address_sget_printable (addr, &s));
+  return s;
 }
 
-int
-emailcmp (char *pattern, char *name)
+char const *
+mh_my_host (void)
 {
-  char *p;
+  mu_address_t addr = mh_local_mailbox ();
+  char const *s;
+  MU_ASSERT (mu_address_sget_domain (addr, 1, &s));
+  return s;
+}
+
+enum part_match_mode
+  {
+    part_match_local,
+    part_match_domain
+  };
 
-  p = strchr (pattern, '@');
-  if (p)
-    for (p++; *p; p++)
-      *p = mu_toupper (*p);
+enum part_match_result
+  {
+    part_match_false,
+    part_match_true,
+    part_match_abort
+  };
 
-  return fnmatch (pattern, name, 0);
+static int match_char_class (char const **pexpr, char c, int icase);
+
+static enum part_match_result
+part_match (char const *expr, char const *name, enum part_match_mode mode)
+{
+  int c;
+
+  while (*expr)
+    {
+      if (*name == 0 && *expr != '*')
+	return part_match_abort;
+      switch (*expr)
+	{
+	case '*':
+	  while (*++expr == '*')
+	    ;
+	  if (*expr == 0)
+	    return part_match_true;
+	  while (*name)
+	    {
+	      int res = part_match (expr, name++, mode);
+	      if (res != part_match_false)
+		return res;
+	    }
+	  return part_match_abort;
+
+	case '?':
+	  expr++;
+	  if (*name == 0)
+	    return part_match_false;
+	  name++;
+	  break;
+	  
+	case '[':
+	  if (!match_char_class (&expr, *name, mode == part_match_domain))
+	    return part_match_false;
+	  name++;
+	  break;
+
+	case '\\':
+	  if (expr[1])
+	    {
+	      c = *++expr; expr++;
+	      if (*name != mu_wordsplit_c_unquote_char (c))
+		return part_match_false;
+	      name++;
+	      break;
+	    }
+	  /* fall through */
+
+	default:
+	  if (mode == part_match_local)
+	    {
+	      if (*expr != *name)
+		return part_match_false;
+	      if ('@' == *name)
+		mode = part_match_domain;
+	    }
+	  else
+	    {
+	      if (mu_tolower (*expr) != mu_tolower (*name))
+		return part_match_false;
+	    }
+	  expr++;
+	  name++;
+	}
+    }
+
+  if (*name == 0)
+    return part_match_true;
+  
+  if (mode == part_match_local && *name == '@')
+    return part_match_true;
+
+  return part_match_false;
+}
+
+static int
+match_char_class (char const **pexpr, char c, int icase)
+{
+  int res;
+  int rc;
+  char const *expr = *pexpr;
+
+  if (icase)
+    c = mu_toupper (c);
+
+  expr++;
+  if (*expr == '^')
+    {
+      res = 0;
+      expr++;
+    }
+  else
+    res = 1;
+
+  if (*expr == '-' || *expr == ']')
+    rc = c == *expr++;
+  else
+    rc = !res;
+
+  for (; *expr && *expr != ']'; expr++)
+    {
+      if (rc == res)
+	{
+	  if (*expr == '\\' && expr[1] == ']')
+	    expr++;
+	}
+      else if (expr[1] == '-')
+	{
+	  if (*expr == '\\')
+	    rc = *++expr == c;
+	  else
+	    {
+	      if (icase)
+		rc = mu_toupper (*expr) <= c && c <= mu_toupper (expr[2]);
+	      else
+		rc = *expr <= c && c <= expr[2];
+	      expr += 2;
+	    }
+	}
+      else if (*expr == '\\' && expr[1] == ']')
+	rc = *++expr == c;
+      else if (icase)
+	rc = mu_toupper(*expr) == c;
+      else
+	rc = *expr == c;
+    }
+  *pexpr = *expr ? expr + 1 : expr;
+  return rc == res;
+}
+
+static int
+email_match (char const *pattern, char const *name)
+{
+  return part_match (pattern, name, part_match_local) == part_match_true;
 }
 
 int
 mh_is_my_name (const char *name)
 {
-  char *pname, *p;
-  int rc = 0;
+  static mu_address_t addr;
+  mu_address_t p;
   
-  pname = mu_strdup (name);
-  p = strchr (pname, '@');
-  if (p)
-    for (p++; *p; p++)
-      *p = mu_toupper (*p);
-  
-  if (emailcmp (mh_my_email (), pname) == 0)
-    rc = 1;
-  else
+  if (!addr)
     {
-      const char *nlist = mh_global_profile_get ("Alternate-Mailboxes", NULL);
+      const char *nlist;
+      int rc;
+      
+      rc = mu_address_create (&addr, mh_my_email ());
+      if (rc)
+	{
+	  mu_diag_funcall (MU_DIAG_ERROR, "mu_address_create", mh_my_email (),
+			   rc);
+	  return 0;
+	}
+      
+      nlist = mh_global_profile_get ("Alternate-Mailboxes", NULL);
       if (nlist)
 	{
-	  const char *end, *p;
-	  char *pat;
-	  int len;
-	  
-	  for (p = nlist; rc == 0 && *p; p = end)
+	  mu_address_t tmp;
+	  struct mu_address hint;
+
+	  hint.domain = NULL;
+	  rc = mu_address_create_hint (&tmp, nlist, &hint,
+				       MU_ADDR_HINT_DOMAIN);
+	  if (rc == 0)
 	    {
-	      
-	      while (*p && mu_isspace (*p))
-		p++;
-
-	      end = strchr (p, ',');
-	      if (end)
-		{
-		  len = end - p;
-		  end++;
-		}
-	      else
-		{
-		  len = strlen (p);
-		  end = p + len;
-		}
-
-	      while (len > 0 && mu_isspace (p[len-1]))
-		len--;
-
-	      pat = mu_alloc (len + 1);
-	      memcpy (pat, p, len);
-	      pat[len] = 0;
-	      rc = emailcmp (pat, pname) == 0;
-	      free (pat);
+	      rc = mu_address_union (&addr, tmp);
+	      if (rc)
+		mu_diag_funcall (MU_DIAG_ERROR, "mu_address_union", NULL, rc);
+	      mu_address_destroy (&tmp);
+	    }
+	  else
+	    {
+	      mu_error (_("bad Alternate-Mailboxes: %s; please fix"),
+			mu_strerror (rc));
 	    }
 	}
     }
-  free (pname);
-  return rc;
+
+  for (p = addr; p; p = p->next)
+    {
+      if (email_match (p->email, name))
+	return 1;
+    }
+  return 0;
 }
 
 static int
@@ -319,7 +394,7 @@ mh_check_folder (const char *pathname, int confirm)
 }
 
 int
-mh_interactive_mode_p ()
+mh_interactive_mode_p (void)
 {
   static int interactive = -1;
 
@@ -384,16 +459,17 @@ mh_getyn_interactive (const char *fmt, ...)
   return rc;
 }
 	    
-FILE *
+mu_stream_t
 mh_audit_open (char *name, mu_mailbox_t mbox)
 {
-  FILE *fp;
+  mu_stream_t str;
   char date[64];
   time_t t;
   struct tm *tm;
   mu_url_t url;
   char *namep;
-  
+  int rc;
+    
   namep = mu_tilde_expansion (name, MU_HIERARCHY_DELIMITER, NULL);
   if (strchr (namep, MU_HIERARCHY_DELIMITER) == NULL)
     {
@@ -402,10 +478,10 @@ mh_audit_open (char *name, mu_mailbox_t mbox)
       namep = p;
     }
 
-  fp = fopen (namep, "a");
-  if (!fp)
+  rc = mu_file_stream_create (&str, namep, MU_STREAM_CREAT|MU_STREAM_APPEND);
+  if (rc)
     {
-      mu_error (_("cannot open audit file %s: %s"), namep, strerror (errno));
+      mu_error (_("cannot open audit file %s: %s"), namep, strerror (rc));
       free (namep);
       return NULL;
     }
@@ -416,17 +492,17 @@ mh_audit_open (char *name, mu_mailbox_t mbox)
   mu_strftime (date, sizeof date, "%a, %d %b %Y %H:%M:%S %Z", tm);
   mu_mailbox_get_url (mbox, &url);
   
-  fprintf (fp, "<<%s>> %s %s\n",
-	   mu_program_name,
-	   date,
-	   mu_url_to_string (url));
-  return fp;
+  mu_stream_printf (str, "<<%s>> %s %s\n",
+		    mu_program_name,
+		    date,
+		    mu_url_to_string (url));
+  return str;
 }
 
 void
-mh_audit_close (FILE *fp)
+mh_audit_close (mu_stream_t str)
 {
-  fclose (fp);
+  mu_stream_close (str);
 }
 
 int
@@ -464,7 +540,7 @@ mh_open_folder (const char *folder, int flags)
 }
 
 char *
-mh_get_dir ()
+mh_get_dir (void)
 {
   const char *mhdir = mh_global_profile_get ("Path", "Mail");
   char *mhcopy;
@@ -885,7 +961,7 @@ mh_set_reply_regex (const char *str)
   if (rc)
     mu_error ("reply_regex: %s%s%s", mu_strerror (rc),
 	      err ? ": " : "",
-	      err ? err : "");
+	      mu_prstr (err));
 }
 
 const char *
@@ -912,7 +988,7 @@ mh_charset (const char *dfl)
 }
 
 int
-mh_decode_2047 (char *text, char **decoded_text)
+mh_decode_2047 (char const *text, char **decoded_text)
 {
   const char *charset = mh_charset (NULL);
   if (!charset)
@@ -1058,4 +1134,13 @@ mh_safe_make_file_name (const char *dir, const char *file)
   return name;
 }
 
+int
+mh_width (void)
+{
+  struct winsize ws;
+  ws.ws_col = ws.ws_row = 0;
+  if ((ioctl(1, TIOCGWINSZ, (char *) &ws) < 0) || ws.ws_col == 0)
+    return 80;  /* FIXME: Should we exit()/abort() if col <= 0 ?  */
+  return ws.ws_col;
+}
 			  
