@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005-2007, 2009-2012, 2014-2017 Free Software
-   Foundation, Inc.
+   Copyright (C) 2003-2019 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -544,9 +543,10 @@ whom (struct mh_whatnow_env *wh, int argc, char **argv, int *status)
   if (!wh->file)
     mu_error (_("no draft file to display"));
   else
-    mh_whom (wh->file, (argc == 2
-			&& (strcmp (argv[1], "-check") == 0
-			    || strcmp (argv[1], "--check") == 0)));
+    mh_whom_file (wh->file,
+		  (argc == 2
+		   && (strcmp (argv[1], "-check") == 0
+		       || strcmp (argv[1], "--check") == 0)));
   return 0;
 }
 
